<?php

namespace Drupal\janrain_connect_ui\Form;

use Drupal\janrain_connect_ui\Service\JanrainConnectUiUsers;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\janrain_connect\Constants\JanrainConnectWebServiceConstants;
use Drupal\janrain_connect_ui\Event\JanrainConnectUiEvents;
use Drupal\janrain_connect_ui\Event\JanrainConnectUiSubmitEvent;
use Drupal\janrain_connect_ui\Service\JanrainConnectUiFormService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Form for configure messages.
 */
class JanrainConnectUiForm extends FormBase {

  /**
   * JanrainConnectUsers.
   *
   * @var \Drupal\janrain_connect_ui\Service\JanrainConnectUiUsers
   */
  private $janrainUsers;

  /**
   * Users form.
   *
   * @var \Drupal\janrain_connect_ui\Form\JanrainConnectUiUsersForm
   */
  private $janrainConnectUiUsersForm;

  /**
   * Config Factory for Janrain Settings.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  private $config;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Janrain Connect Form Service.
   *
   * @var \Drupal\janrain_connect_ui\Service\JanrainConnectUiFormService
   */
  private $janrainConnectFormService;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactory $config_factory, JanrainConnectUiFormService $janrain_connect_form_service, EventDispatcherInterface $event_dispatcher, JanrainConnectUiUsers $janrain_users) {
    $this->config = $config_factory->get('janrain_connect.settings');
    $this->janrainConnectFormService = $janrain_connect_form_service;
    $this->eventDispatcher = $event_dispatcher;
    $this->janrainUsers = $janrain_users;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('janrain_connect_ui.form'),
      $container->get('event_dispatcher'),
      $container->get('janrain_connect_ui.users')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'janrain_connect_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $form_id = '') {

    // We need check if is test and don't call janrain service.
    // @codingStandardsIgnoreLine
    $path = \Drupal::service('path.current')->getPath();
    $path = explode('/', $path);

    // Default value.
    $data_for_tests = FALSE;

    if (!empty(end($path) && end($path) == 'test')) {
      $data_for_tests = TRUE;
    }

    $form_data = $this->janrainConnectFormService->getForm($form_id, $data_for_tests);

    if (empty($form_data)) {
      throw new NotFoundHttpException();
    }

    // Check form status.
    self::checkFormStatus($form_id);

    $form_state->set('form_id', $form_id);

    $fields = isset($form_data['fields']) ? $form_data['fields'] : [];

    foreach ($fields as $key => $field) {

      $field_id = $key;

      $label = $field['label'];
      $type = 'textfield';
      $required = FALSE;
      $options = FALSE;

      if (!empty($field['validations']['required'])) {
        $required = $field['validations']['required'];
      }

      $type = $field['type'];
      $field_mapping = $this->config->get('field_mapping');

      if (!empty($field_mapping[$type])) {
        $type = $field_mapping[$type];
      }

      $options = $field['options'];

      $classes = $field['classes'];

      $form[$field_id] = [
        '#type' => $type,
        '#title' => $label,
        '#required' => $required,
      ];

      if ($options) {
        $form[$field_id]['#options'] = $options;
      }

      if ($classes) {
        $form[$field_id]['#attributes']['class'] = $classes;
      }

      if ($type === 'date') {
        $form[$field_id]['#attributes']['type'] = $type;
      }

      if ($field['placeholder']) {
        $form[$field_id]['#attributes']['placeholder'] = $field['placeholder'];
      }

      if ($field['description']) {
        $form[$field_id]['#description'] = $field['description'];
      }
    }

    $submit_name = $this->t('Submit');

    $config_form_data = $this->config->get('config_form_data');

    if (!empty($config_form_data[$form_id]['submit_button'])) {
      // Submit button should be translatable @codingStandardsIgnoreLine.
      $submit_name = $this->t($config_form_data[$form_id]['submit_button']);
    }

    // Fill default values.
    $this->fillDefaultValues($form, $form_id);

    // Set Read Only Fields.
    $this->setReadOnlyFields($form, $form_id);

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $submit_name,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $this->submitFormToJanrain($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitFormToJanrain(array &$form, FormStateInterface $form_state) {

    $form_id = '';
    $data = [];
    $drupal_internal_form_keys = [
      'submit',
      'form_build_id',
      'form_token',
      'form_id',
      'op',
    ];

    if (!empty($form_state->get('form_id'))) {
      $form_id = $form_state->get('form_id');
    }

    if (!empty($form_state->getValues())) {

      $data = $form_state->getValues();

      // Remove Drupal internal form keys.
      foreach ($drupal_internal_form_keys as $key) {
        unset($data[$key]);
      }
    }

    foreach ($data as $key => $current_data) {

      if (empty($form[$key]['#attributes']['class'])) {
        continue;
      }

      $classes = $form[$key]['#attributes']['class'];

      if (!empty($current_data) && in_array('dateselect', $classes)) {

        $field_date_select = $current_data;

        $field_date_select_array = explode("-", $field_date_select);

        $dateselect_day = $field_date_select_array[2];
        $dateselect_month = $field_date_select_array[1];
        $dateselect_year = $field_date_select_array[0];

        $data[$key] = [
          JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FIELD_DATESELECT_DAY => $dateselect_day,
          JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FIELD_DATESELECT_MONTH => $dateselect_month,
          JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FIELD_DATESELECT_YEAR => $dateselect_year,
        ];
      }
    }

    if (empty($data['birthdate'])) {
      unset($data['birthdate']);
    }

    $event = new JanrainConnectUiSubmitEvent($form_id, $data, $form, $form_state);

    $this->eventDispatcher->dispatch(JanrainConnectUiEvents::EVENT_SUBMIT, $event);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {}

  /**
   * Method to fill default values.
   */
  public function fillDefaultValues(&$form, $form_id) {

    switch ($form_id) {

      case JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_EDIT_PROFILE;
        $this->janrainUsers->fillUserValuesUpdateProfileForm($form, $form_id);
        break;

      default:
        break;
    }
  }

  /**
   * Method to set Read Only Fields.
   */
  public function setReadOnlyFields(&$form, $form_id) {

    $readonly_fields = $this->config->get('readonly_fields');

    if (empty($readonly_fields[$form_id])) {
      return FALSE;
    }

    foreach ($readonly_fields[$form_id] as $field_key) {
      $form[$field_key]['#attributes'] = ['disabled' => 'disabled'];
    }
  }

  /**
   * Method to check Form Status.
   */
  public function checkFormStatus($form_id, $show_message = TRUE) {

    if (empty($this->config->get('disabled_forms'))) {
      return TRUE;
    }

    $disabled_forms = $this->config->get('disabled_forms');

    if (in_array($form_id, $disabled_forms)) {

      if ($show_message) {

        drupal_set_message($this->t('The form: @form may not work correctly because it is not fully implemented.', [
          '@form' => $form_id,
        ]), 'warning', TRUE);

        drupal_set_message($this->t('For more information access our <a href="@url_documentation" target="blank">official Janrain documentation</a>.', [
          '@url_documentation' => 'https://www.drupal.org/docs/8/modules/drupal-connector-for-janrain-identity-cloud/forms-as-block',
        ]), 'warning', TRUE);

      }

      return FALSE;

    }
  }

}
