<?php

namespace Drupal\janrain_connect\Service;

use Drupal\Core\Url;
use Drupal\janrain_connect\Constants\JanrainConnectWebServiceConstants;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Component\Serialization\Json;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Config\ConfigFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * JanrainConnect Flow Class.
 */
class JanrainConnectConnector {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The HTTP client used to fetch remote definitions.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The service container.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface
   */
  protected $container;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $janrainConnectValidate;

  /**
   * JanrainConnector constructor.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Container.
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   Config Factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   Http Client.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language Manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Logger Factory.
   * @param \Drupal\janrain_connect\Service\JanrainConnectValidate $janrain_connect_validate
   *   JanrainConnect validate.
   */
  public function __construct(
  ContainerInterface $container,
  ConfigFactory $config_factory,
  ClientInterface $http_client,
  LanguageManagerInterface $language_manager,
  LoggerChannelFactoryInterface $logger_factory,
  JanrainConnectValidate $janrain_connect_validate
  ) {
    $this->config = $config_factory->get('janrain_connect.settings');
    $this->container = $container;
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
    $this->languageManager = $language_manager;
    $this->logger = $logger_factory->get('janrain_connect');
    $this->janrainConnectValidate = $janrain_connect_validate;
  }

  /**
   * Get Service By Form.
   *
   * @param string $form_id
   *   The Form ID.
   *
   * @return mixed
   *   Return string with the Service URL or FALSE.
   */
  public function getServiceByForm($form_id) {

    $mapping_form_services = $this->config->get('mapping_form_services');

    if (empty($mapping_form_services[$form_id]) || !is_string($mapping_form_services[$form_id])) {
      return FALSE;
    }

    $service_url = $mapping_form_services[$form_id];

    return $service_url;
  }

  /**
   * Perform Janrain Calls.
   *
   * @param string $service_url
   *   Service URL.
   * @param array $data
   *   Data.
   * @param string $method
   *   Method.
   * @param string $requestType
   *   Request Type.
   * @param string $clientId
   *   Client ID.
   * @param string $clientSecret
   *   Client Secret.
   *
   * @return mixed
   *   Return Janrain response.
   */
  private function janrainCalls($service_url, array $data, $method, $requestType = NULL, $clientId = NULL, $clientSecret = NULL) {

    $headers = [
      'Accept' => 'application/json',
      'Content-Type' => 'application/json',
    ];

    $auth = [];

    // Compatible on PHP 7.1. Will be updated by lib. @codingStandardsIgnoreLine
    $capture_server_url = \Drupal::config('janrain_connect.settings')->get('capture_server_url');

    // HTTP Request Guzzle. Will be updated by lib. @codingStandardsIgnoreLine
    $client = \Drupal::httpClient();

    if (!is_null($clientId) && !is_null($clientSecret)) {
      $auth = [$clientId => $clientSecret];
    }

    $body = json_encode($data);

    // Compatible on PHP 7.1. Will be updated by lib. @codingStandardsIgnoreLine
    \Drupal::logger('janrain_connect')->notice('[STARTED][PID: @my_pid@]] Call to Janrain service: @service_url@ at @time@', [
      '@my_pid@' => getmypid(),
      '@service_url@' => $service_url,
      '@time@' => time(),
    ]);

    $response = $client->$method($capture_server_url . $service_url, [
      'auth' => $auth,
      'body' => $body,
      'headers' => $headers,
    ]);

    // Compatible on PHP 7.1. Will be updated by lib. @codingStandardsIgnoreLine
    \Drupal::logger('janrain_connect')->notice('[FINISHED][PID: @my_pid@] Call to Janrain service: @service_url@ at @time@', [
      '@my_pid@' => getmypid(),
      '@service_url@' => $service_url,
      '@time@' => time(),
    ]);

    $result = $response->getBody()->getContents();

    if (empty($result)) {
      return FALSE;
    }

    $result = Json::decode($result);

    // Validate response.
    $this->janrainConnectValidate->validateResponse($result);

    return $result;
  }

  /**
   * Register User on Janrain.
   *
   * @param array $data
   *   Array with Data.
   * @param string $form_id
   *   The Form ID.
   * @param string $locale
   *   Locale with language.
   * @param string $redirect_uri
   *   Redirect URL.
   *
   * @return mixed
   *   Return janrain result using janrainCalls.
   */
  public function register(array $data, $form_id, $locale = NULL, $redirect_uri = NULL) {

    if (!$redirect_uri) {
      $redirect_uri = Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString();
    }

    $client_id = $this->config->get('client_id');
    $flow_name = $this->config->get('flow_name');
    $flow_version = $this->config->get('flow_version');
    $entity_type = $this->config->get('entity_type');

    $service_url = $this->getServiceByForm($form_id);

    if (!$locale) {

      $locale = $this->config->get('default_language');

      $language = $this->languageManager->getCurrentLanguage();
      $lid = $language->getId();

      $flow_language_key = 'flow_language_mapping_' . $lid;

      if (!empty($this->config->get($flow_language_key))) {
        $locale = $this->config->get($flow_language_key);
      }
    }

    $data['client_id'] = $client_id;
    $data['flow'] = $flow_name;
    $data['flow_version'] = $flow_version;
    $data['locale'] = $locale;
    $data['redirect_uri'] = $redirect_uri;
    $data['response_type'] = JanrainConnectWebServiceConstants::JANRAIN_CONNECT_CODE_AND_TOKEN;
    $data['form'] = JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_REGISTRATION;

    if (!empty($entity_type)) {
      $data['type_name'] = $entity_type;
    }

    $method = 'post';

    return $this->janrainCalls($service_url, $data, $method);
  }

  /**
   * Activation Account.
   *
   * @param string $verificationCode
   *   Verification Code.
   */
  public function activationAccount($verificationCode) {
    $serviceUrl = '/access/useVerificationCode';

    $data = [
      'verification_code' => $verificationCode,
    ];

    $method = 'get';

    $requestType = 'multipart';

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType);
  }

  /**
   * Get Access Token with UUID.
   *
   * @param string $uuid
   *   UUID.
   */
  public function getAccessTokenWithUuid($uuid) {
    $serviceUrl = '/access/getAccessToken';

    $data = [
      'uuid' => $uuid,
      'type_name' => 'user',
    ];

    $method = 'get';

    $requestType = 'multipart';

    $clientId = $this->config->get('full_client_id');
    $clientSecret = $this->config->get('full_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Reset user password.
   *
   * @param string $newPassword
   *   Password.
   * @param string $accessToken
   *   Access Token.
   * @param string $locale
   *   Locale.
   */
  public function resetPassword($newPassword, $accessToken, $locale = 'en-US') {
    $serviceUrl = '/oauth/update_profile_native';

    $data = [
      'client_id' => $this->config->get('login_client_id'),
      'flow' => $this->config->get('flow'),
      'flow_version' => $this->config->get('flow_version'),
      'locale' => $locale,
      'form' => 'changePasswordFormNoAuth',
      'newPassword' => $newPassword,
      'newPasswordConfirm' => $newPassword,
      'access_token' => $accessToken,
    ];

    $method = 'post';

    $requestType = 'multipart';

    $clientId = $this->config->get('login_client_id');

    $clientSecret = $this->config->get('login_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Login User on Janrain.
   *
   * @param string $email
   *   Email.
   * @param string $password
   *   Password.
   * @param string $locale
   *   Locale.
   */
  public function login($email, $password, $locale = 'en-US') {

    if (empty($this->getServiceByForm(JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_SIGNIN))) {
      return FALSE;
    }

    $service_url = $this->getServiceByForm(JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_SIGNIN);

    $data = [
      'client_id' => $this->config->get('client_id'),
      'flow' => $this->config->get('flow_name'),
      'flow_version' => $this->config->get('flow_version'),
      'locale' => $locale,
      'redirect_uri' => $this->config->get('app_url'),
      'response_type' => 'code_and_token',
      'form' => 'signInForm',
      'signInEmailAddress' => $email,
      'currentPassword' => $password,
    ];

    $method = 'post';

    $requestType = 'multipart';

    return $this->janrainCalls($service_url, $data, $method, $requestType);
  }

  /**
   * Provide Access and Renew Tokens after login or register.
   *
   * @param string $authorizationCode
   *   Authorization Code.
   * @param string $redirectUri
   *   Redirect URI.
   */
  public function getTokens($authorizationCode, $redirectUri) {
    $serviceUrl = '/oauth/token';

    $data = [
      'grant_type' => 'authorization_code',
      'code' => $authorizationCode,
      'redirect_uri' => $redirectUri,
    ];

    $method = 'post';

    $requestType = 'form';

    $clientId = $this->config->get('login_client_id');
    $clientSecret = $this->config->get('login_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Get user data from Entity Api.
   *
   * @param string $token
   *   Access Token.
   */
  public function getUserData($token) {

    // We will use Janrain lib for get service_url @codingStandardsIgnoreLine
    $service_url = '/entity';

    $data = [
      'type_name' => 'user',
      'access_token' => $token,
    ];

    $params = http_build_query($data);

    $service_url .= '?' . $params;

    $method = 'get';

    return self::janrainCalls($service_url, [], $method);
  }

  /**
   * Get new access token.
   *
   * @param string $refreshToken
   *   Refresh Token.
   */
  public function refreshToken($refreshToken) {

    $serviceUrl = '/oauth/token';

    $data = [
      'grant_type' => 'refresh_token',
      'refresh_token' => $refreshToken,
    ];

    $method = 'get';

    $requestType = 'multipart';
    $clientId = $this->config->get('login_client_id');
    $clientSecret = $this->config->get('login_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Get the janrain user using his email.
   *
   * @param string $email
   *   Email.
   */
  public function getUserByEmail($email) {

    $serviceUrl = '/entity.find';

    $data = [
      'type_name' => 'user',
      'filter' => "email = '" . $email . "'",
    ];

    $method = 'get';

    $requestType = 'multipart';
    $clientId = $this->config->get('full_client_id');
    $clientSecret = $this->config->get('full_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Janrain Forgot Password.
   *
   * @param string $email
   *   Email.
   * @param string $form_id
   *   Form ID.
   * @param string $locale
   *   Locale.
   * @param string $redirect_uri
   *   Redirect URI.
   */
  public function forgotPassword($email, $form_id = NULL, $locale = NULL, $redirect_uri = NULL) {

    // Get service_url from mapping.
    if (!$form_id || empty($this->getServiceByForm($form_id))) {
      return FALSE;
    }

    $service_url = $this->getServiceByForm($form_id);

    if (!$redirect_uri) {
      $redirect_uri = Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString();
    }

    if (!$locale) {

      $locale = $this->config->get('default_language');

      $language = $this->languageManager->getCurrentLanguage();
      $lid = $language->getId();

      $flow_language_key = 'flow_language_mapping_' . $lid;

      if (!empty($this->config->get($flow_language_key))) {
        $locale = $this->config->get($flow_language_key);
      }
    }

    $data = [
      'client_id' => $this->config->get('client_id'),
      'flow' => $this->config->get('flow_name'),
      'flow_version' => $this->config->get('flow_version'),
      'locale' => $locale,
      'redirect_uri' => $redirect_uri,
      'form' => 'forgotPasswordForm',
      'signInEmailAddress' => $email,
    ];

    $method = 'post';

    $request_type = 'multipart';

    return $this->janrainCalls($service_url, $data, $method, $request_type);
  }

  /**
   * Get Verification Code.
   *
   * @param string $uuid
   *   UUID.
   */
  public function getVerificationCode($uuid) {
    $serviceUrl = '/access/getVerificationCode';

    $data = [
      'uuid' => $uuid,
      'type_name' => 'user',
      'attribute_name' => 'emailVerified',
    ];

    $method = 'get';

    $requestType = 'multipart';

    $clientId = $this->config->get('full_client_id');
    $clientSecret = $this->config->get('full_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Resend Verify Email.
   *
   * @param string $email
   *   Email.
   * @param string $locale
   *   Locale.
   */
  public function resendVerifyEmail($email, $locale) {
    $serviceUrl = '/oauth/verify_email_native';

    $data = [
      'client_id' => $this->config->get('login_client_id'),
      'flow' => $this->config->get('flow'),
      'flow_version' => $this->config->get('flow_version'),
      'locale' => $locale,
      'redirect_uri' => $this->config->get('app_url') . '/activation',
      'form' => 'resendVerificationForm',
      'signInEmailAddress' => $email,
    ];

    $method = 'post';

    $requestType = 'multipart';
    $clientId = $this->config->get('login_client_id');
    $clientSecret = $this->config->get('login_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Change Password.
   *
   * @param string $actual_password
   *   Actual Password.
   * @param string $new_password
   *   New Password.
   * @param string $new_password_confirm
   *   Confirmation for New Password.
   * @param string $token
   *   Access Token.
   * @param string $locale
   *   Locale.
   * @param string $form_id
   *   Form ID.
   */
  public function changePassword($actual_password, $new_password, $new_password_confirm, $token, $locale = NULL, $form_id = NULL) {

    // Get service_url from mapping.
    if (!$form_id || empty($this->getServiceByForm($form_id))) {
      return FALSE;
    }

    $service_url = $this->getServiceByForm($form_id);

    if (!$locale) {

      $locale = $this->config->get('default_language');

      $language = $this->languageManager->getCurrentLanguage();
      $lid = $language->getId();

      $flow_language_key = 'flow_language_mapping_' . $lid;

      if (!empty($this->config->get($flow_language_key))) {
        $locale = $this->config->get($flow_language_key);
      }
    }

    $data = [
      'client_id' => $this->config->get('client_id'),
      'flow' => $this->config->get('flow_name'),
      'flow_version' => $this->config->get('flow_version'),
      'locale' => $locale,
      'currentPassword' => $actual_password,
      'form' => $form_id,
      'newPassword' => $new_password,
      'newPasswordConfirm' => $new_password_confirm,
      'access_token' => $token,
    ];

    $method = 'post';

    $request_type = 'multipart';

    return $this->janrainCalls($service_url, $data, $method, $request_type);
  }

  /**
   * Update the last update password flag.
   *
   * @param string $uuid
   *   UUID.
   */
  public function updateLastUpdatePassword($uuid) {
    $serviceUrl = '/entity.update';

    $data = [
      'value' => '{"ciandt_last_update_password":"' . date('Y-m-d H:i:s', time()) . '"}',
      'type_name' => 'user',
      'uuid' => $uuid,
    ];

    $method = 'get';

    $requestType = 'multipart';
    $clientId = $this->config->get('full_client_id');
    $clientSecret = $this->config->get('full_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Generic Janrain Calls.
   *
   * @param string $serviceUrl
   *   Service URL.
   * @param string $data
   *   Data.
   * @param string $method
   *   Method.
   * @param string $requestType
   *   Request Type.
   * @param string $clientType
   *   Client Type.
   */
  public function genericJanrainCalls($serviceUrl, $data, $method, $requestType, $clientType = NULL) {
    $clientId = NULL;
    $clientSecret = NULL;

    if (!is_null($clientType)) {
      switch ($clientType) {
        case 'login_client':
          $clientId = $this->config->get('login_client_id');
          $clientSecret = $this->config->get('login_client_secret');
          break;

        case 'full_access_client':
          $clientId = $this->config->get('full_client_id');
          $clientSecret = $this->config->get('full_client_secret');
          break;
      }
    }

    if (!empty($clientType) && $method == 'POST') {
      $data['client_id'] = $clientId;
      $data['flow'] = $this->config->get('flow');
      $data['flow_version'] = $this->config->get('flow_version');
    }

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Get Authorization Code with UUID.
   *
   * @param string $uuid
   *   UUID.
   */
  public function getAuthorizationCodeWithUuid($uuid) {

    $serviceUrl = '/access/getAuthorizationCode';

    $data = [
      'uuid' => $uuid,
      'type_name' => 'user',
      'redirect_uri' => $this->config->get('app_url'),
      'for_client_id' => $this->config->get('login_client_id'),
    ];

    $method = 'get';

    $requestType = 'multipart';

    $clientId = $this->config->get('full_client_id');
    $clientSecret = $this->config->get('full_client_secret');

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Get settings from Janrain.
   *
   * @param string $client
   *   Client.
   */
  public function getClientSettings($client) {

    $clientId = $this->config->get('full_client_id');
    $clientSecret = $this->config->get('full_client_secret');

    if ($client == 'login') {
      $clientId = $this->config->get('login_client_id');
      $clientSecret = $this->config->get('login_client_secret');
    }

    $serviceUrl = '/settings/items';

    $data = [];

    $method = 'get';

    $requestType = 'multipart';

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId, $clientSecret);
  }

  /**
   * Update a user profile in Janrain.
   *
   * @param string $accessToken
   *   Access token provided when user is logged in.
   * @param array $data
   *   User data to be updated.
   * @param string $locale
   *   User locale.
   *
   * @return array
   *   Janrain response.
   */
  public function updateUserProfile($accessToken, array $data, $locale = 'en-US') {
    $serviceUrl = '/oauth/update_profile_native';

    $clientId = $this->config->get('client_id');

    $data['client_id'] = $clientId;
    $data['flow'] = $this->config->get('flow_name');
    $data['flow_version'] = $this->config->get('flow_version');
    $data['locale'] = $locale;
    $data['form'] = JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_EDIT_PROFILE;
    $data['access_token'] = $accessToken;

    $method = 'post';

    $requestType = 'multipart';

    return $this->janrainCalls($serviceUrl, $data, $method, $requestType, $clientId);
  }

}
